function installSensorGloveV2()

    %% Helper functions
    function tf = userChoice(question,default,fun)
        if(default)
            choices = 'Y/n';
            defChoice= 'Y';
        else
            choices = 'y/N';
            defChoice= 'N';
        end
        done = false;
        while(~done)
            choice = input(sprintf('%s %s: ',question,choices),'s');
            if(isempty(choice))
                choice = defChoice;
            end
            if(strcmpi(choice(1),'y'))
                fun();
                done = true;
                tf = 1;
            elseif(strcmpi(choice(1),'n'))
                done = true;
                tf = 0;
            else
                fprintf('invalid input\n');
            end
        end
        
    end

    function updateFile(fileName, pattern, newLine)
        
        if(exist(fileName,'file')==2)
            fid = fopen(fileName,'r');
            entries = textscan(fid,'%s','Delimiter','\n');
            fclose(fid);
            entries = entries{1}(cellfun(@isempty,regexp(entries{1},pattern)));
        else
            entries = {};
        end

        entries{end+1} = newLine;

        fid = fopen(fileName, 'w');
        fwrite(fid, strjoin(entries(:)','\n'));
        fclose(fid);
    end

    %% Welcome
    fprintf('\n\nYou are about to install the SensorGloveV2 Matlab Interface\n\n\n');

    %% Adding wrappers to matlab path
    m = fileparts(mfilename('fullpath'));
    mFile = '~/.profile';
    mPattern = '^export MATLABPATH=\$MATLABPATH:.*SensorGloveV2/matlab$';
    mLine = ['export MATLABPATH=$MATLABPATH:',m];
    if(userChoice(sprintf('Would you like to add ''%s'' to the Matlab path?',m),1,@()updateFile(mFile,mPattern,mLine)))
        fprintf('!!! Changes will take effect the next time you log in !!!\n');
    end
    fprintf('\n\n');
    
    %% Adding jars to matlab classpath
    j = [fileparts(fileparts(mfilename('fullpath'))),filesep,'jar',filesep,'SensorGloveV2.jar'];
    jFile = [prefdir,filesep,'javaclasspath.txt'];
    jPattern = '^.*SensorGloveV2.jar$';
    jLine = j;
    if(userChoice(sprintf('Would you like to add ''%s'' to the Matlab java classpath?',j),1,@()updateFile(jFile,jPattern,jLine)))
        fprintf('!!! Changes will take effect the next time you start Matlab !!!\n');
    end
    fprintf('\n\n');

    %% Adding serial port acces to java.opts
    javaOpts='-Dgnu.io.rxtx.SerialPorts=/dev/ttyS0:/dev/ttyS1:/dev/USB0:/dev/ttyACM0:/dev/ttyACM1:/dev/rfcomm0';
    javaOptsFile=[matlabroot,filesep,'bin',filesep,computer('arch'),filesep,'java.opts'];
    if(userChoice(sprintf('Would you like to add the option\n%s\nto ''%s'' ? (sudo rights required)',javaOpts,javaOptsFile),1,@()system(sprintf('sudo bash -c ''echo %s >> %s''',javaOpts,javaOptsFile))))
        fprintf('!!! Changes will take effect the next time you start Matlab !!!\n');
    end
    fprintf('\n\n');
    
    %% Adding user to dialout
    [~, user] = system('id -un');
    user = strtrim(user);
    if(userChoice(sprintf('Would you like to add the user ''%s'' to the group ''dialout''? (sudo rights required)',user),1,@()system(sprintf('sudo adduser %s dialout',user))))
        fprintf('!!! Changes will take effect the next time you log in !!!\n');
    end
    fprintf('\n\n');
    
    %% Godbye
    fprintf('Installation complete.\nYou can test the SensorGloveV2 bye executing the ''testSensorGloveV2'' function.\n')
end
